/*
 * $Header: /var/cvsroot/Slide/src/org/apache/webdav/lib/methods/EdealPropPatchMethod.java,v 1.4 2010/02/03 09:30:57 brian Exp $
 * $Revision: 1.4 $
 * $Date: 2010/02/03 09:30:57 $
 *
 * ====================================================================
 *
 * Copyright 1999-2002 The Apache Software Foundation 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package org.apache.webdav.lib.methods;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpConnection;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpState;
import org.apache.webdav.lib.util.EDealXMLPrinter;
import org.apache.webdav.lib.util.WebdavStatus;
import org.apache.webdav.lib.util.XMLPrinter;


/**
 * PROPPATCH Method.
 *
 */
public class EdealPropPatchMethod
    extends XMLResponseMethodBase {


	private String parsedBody = null;
    // ----------------------------------------------------------- Constructors


    /**
     * Method constructor.
     */
    public EdealPropPatchMethod() {
    	init();
    }


    /**
     * Method constructor.
     */
    public EdealPropPatchMethod(String path) {
        super(path);
        
        init();
    }

    private void init() {
    	namespaces.put("DAV:","DAV");
    	namespaces.put("urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/","dt");    	
    }

    // ----------------------------------------------------- Instance Variables


    /**
     * Hashtable of the properties to set.
     */
    protected Hashtable toSet = new Hashtable();


    /**
     * Hashtable of the properties to remove.
     */
    protected Hashtable toRemove = new Hashtable();


    protected Hashtable namespaces = new Hashtable();
    protected Hashtable dataTypes = new Hashtable();
    protected int namespaceCnt = 0;

    // --------------------------------------------------------- Public Methods


    /**
     * Add a new property to set.
     *
     * @param name Property name
     * @param value Property value
     */
    public void addPropertyToSet(String name, String value) {
        checkNotUsed();
        Property propertyToSet = new Property();
        if (name != null) {
            propertyToSet.name = name;
            if (value != null)
                propertyToSet.value = value;
            else
                propertyToSet.value = "";
            toSet.put(name, propertyToSet);
        }
    }


    /**
     * Add a new property to set.
     *
     * @param name Property name
     * @param value Property value
     * @param namespace Namespace abbreviation
     * @param namespaceInfo Namespace information
     */
    public void addPropertyToSet(String name, String value, String namespace,
                                 String namespaceInfo) {
    	
    	checkNotUsed();
        Property propertyToSet = new Property();
        if (name != null) {
            propertyToSet.name = name;
            if (value != null)
                propertyToSet.value = value;
            else
                propertyToSet.value = "";
            propertyToSet.namespace = namespace;
            propertyToSet.namespaceInfo = namespaceInfo;
            toSet.put(namespace + name, propertyToSet);
        }
    }


	/**
	 * Add a new property to set.
	 *
	 * @param name Property name
	 * @param value Property value
	 * @param namespace Namespace abbreviation
	 * @param namespaceInfo Namespace information
	 */
	public void addPropertyToSet(String name, Object value, String namespace,
	                             String namespaceInfo, String type) {
		
		checkNotUsed();
	    Property propertyToSet = new Property();
	    if (name != null) {
	        propertyToSet.name = name;
	        if (value != null)
	            propertyToSet.value = value;
	        else
	            propertyToSet.value = "";
	        propertyToSet.namespace = namespace;
	        propertyToSet.namespaceInfo = namespaceInfo;
	        toSet.put(namespace + name, propertyToSet);
	    }
	    
	    if (namespaces.get(namespaceInfo) == null) {
	    	char nextChar = (char)('A' + namespaceCnt++); 
	    	namespaces.put(namespaceInfo, "" + nextChar);
	    }
	    
	    if (type != null) {
	    	dataTypes.put(namespaceInfo + name, type);
	    }
	}


    /**
     * Add property to remove.
     *
     * @param name Property name
     */
    public void addPropertyToRemove(String name) {
        checkNotUsed();
        Property propertyToRemove = new Property();
        if (name != null) {
            propertyToRemove.name = name;
            toRemove.put(name, propertyToRemove);
        }
    }


    /**
     * Add property to remove.
     *
     * @param name Property name
     * @param namespace Namespace abbreviation
     * @param namespaceInfo Namespace information
     */
    public void addPropertyToRemove(String name, String namespace,
                                    String namespaceInfo) {
        checkNotUsed();
        Property propertyToRemove = new Property();
        if (name != null) {
            propertyToRemove.name = name;
            propertyToRemove.namespace = namespace;
            propertyToRemove.namespaceInfo = namespaceInfo;
            toRemove.put(name, propertyToRemove);
        }
    }


    // --------------------------------------------------- WebdavMethod Methods


    public String getName() {
        return "PROPPATCH";
    }

    /**
     * Generate additional headers needed by the request.
     *
     * @param state State token
     * @param conn the connection
     */
    public void addRequestHeaders(HttpState state, HttpConnection conn)
    throws IOException, HttpException {

        // set the default utf-8 encoding, if not already present
        if (getRequestHeader("Content-Type") == null ) {
        	super.setRequestHeader("Content-Type", "text/xml; charset=utf-8");
        }
        if (getRequestHeader("Content-Length") == null ) {
        	super.setRequestHeader("Content-Length", String.valueOf(this.generateRequestBody().length()));
        }
        super.addRequestHeaders(state, conn);

        Header cl = getRequestHeader("Content-Length");
        if (cl == null) {
        	addRequestHeader("Content-Length",String.valueOf(generateRequestBody().length()));
        }
    }

    /**
     * DAV requests that contain a body must override this function to
     * generate that body.
     *
     * <p>The default behavior simply returns an empty body.</p>
     */
    protected String generateRequestBody() {
    	if (parsedBody != null) {
    		return parsedBody;
    	}
    	
        EDealXMLPrinter printer = new EDealXMLPrinter();

        printer.writeXMLHeader();
        printer.writeEDealHeader(namespaces);
        
        if (toSet.size() > 0) {

            printer.writeElement("DAV", null, "set", XMLPrinter.OPENING);

            Enumeration toSetList = toSet.elements();
            printer.writeElement("DAV", null, "prop", XMLPrinter.OPENING);
            while (toSetList.hasMoreElements()) {
                Property current = (Property) toSetList.nextElement();
            	try {
	                printer.writeEDealProperty((String)namespaces.get(current.namespaceInfo), current.name, current.value, (String) dataTypes.get(current.namespaceInfo + current.name));
            	} catch (Exception e) {
            	}
            }
            	
            printer.writeElement("DAV", null, "prop", XMLPrinter.CLOSING);

            printer.writeElement("DAV", null, "set", XMLPrinter.CLOSING);

        }

        printer.writeElement("DAV", "propertyupdate", XMLPrinter.CLOSING);
        
        String printXml = System.getProperty("com.edeal.synchro.webdav.printXml");
        if (printXml != null && printXml.equalsIgnoreCase("true")) {
        	System.out.println("\n---------------------> To server");
        	System.out.println(printer.toString());
        }
        
        parsedBody = printer.toString();
        return parsedBody;
    }

    /**
     * Parse response.
     *
     * @param input Input stream
     */
    public void parseResponse(InputStream input, HttpState state, HttpConnection conn)
        throws IOException, HttpException {
        try
        {
            int code = getStatusLine().getStatusCode();
            if (code == WebdavStatus.SC_CONFLICT     ||
                code == WebdavStatus.SC_MULTI_STATUS ||
                code == WebdavStatus.SC_FORBIDDEN ) {
                parseXMLResponse(input);
            }
        }
        catch (IOException e) {
                // FIX ME:  provide a way to deliver non xml data
        }
    }


    // --------------------------------------------------- Property Inner Class


    private class Property {

        public String name = "";
        public String namespace;
        public String namespaceInfo;
        public Object value;

    }
}
